@@ -1,14 +1,20 @@
 import sys
 
 from rex.utils.config import ConfigArgument, ConfigParser
-from rex.utils.logging import logger
-from rex.utils.registry import NAMESPACE_REGISTRY, call_register, get_registered
+from rex.utils.registry import NAMESPACE_REGISTRY, get_registered, register
+from rex.version import __version__
 
 from .emb import emb
+from .gen_conf import gen_conf
 from .new import new
 from .train import train
 
 
+@register("rex_init_call")
+def version(*args, **kwargs):
+    print(f"{__version__}")
+
+
 def main():
     args = ConfigParser.parse_cmd(
         ConfigArgument(
@@ -18,8 +24,8 @@ def main():
         ),
         init_priority_args=False,
         cmd_args=sys.argv[1:2],
+        description=f"REx (version: {__version__}) - A toolkit for Relation, Event eXtraction (REx) and more...",
     )
-    logger.info(f"Entering {args.command} mode...")
 
     if args.command in NAMESPACE_REGISTRY["rex_init_call"]:
         command = get_registered("rex_init_call", args.command)
