@@ -8,7 +8,7 @@ def emb(cmd_args=None):
     config = ConfigParser.parse_cmd(
         ConfigArgument("-r", "--raw-emb-filepath", help="raw embedding filepath"),
         ConfigArgument("-o", "--dump-emb-filepath", help="dumped embedding filepath"),
-        ConfigArgument("-f", "--filepaths", nargs="*", help="raw text filepaths"),
+        ConfigArgument("-f", "--filepaths", nargs="+", help="raw text filepaths"),
         cmd_args=cmd_args,
     )
     build_emb(config.raw_emb_filepath, config.dump_emb_filepath, *config.filepaths)
