@@ -2,7 +2,11 @@ from typing import Any, Dict, List, Optional
 
 import torch
 
-from rex.utils.iteration import flatten_all
+from rex.utils.batch import (
+    group_instances_into_batch,
+    validate_instance_has_the_same_keys,
+)
+from rex.utils.iteration import flatten_all_iter
 
 
 class GeneralCollateFn(object):
@@ -23,10 +27,16 @@ class GeneralCollateFn(object):
     DEFAULT_TYPE_MAP = {int: torch.long, float: torch.float, str: None}
 
     def __init__(
-        self, key2type: Optional[dict] = {}, guessing: Optional[bool] = False
+        self,
+        key2type: Optional[Dict[str, Any]],
+        guessing: Optional[bool] = False,
+        missing_key_as_null: Optional[bool] = False,
     ) -> None:
-        self.key2type = key2type
+        self.key2type = {}
+        if isinstance(key2type, dict) and len(key2type) > 0:
+            self.key2type.update(key2type)
         self.guessing = guessing
+        self.missing_key_as_null = missing_key_as_null
 
     def update_type_mapping(self, key2type: dict):
         for key, val_type in key2type.items():
@@ -40,7 +50,7 @@ class GeneralCollateFn(object):
                 key2type[key] = self.DEFAULT_TYPE_MAP[val_type]
             elif val_type == list:
                 item_types = set()
-                for item in flatten_all(val):
+                for item in flatten_all_iter(val):
                     item_types.add(type(item))
                 if len(item_types) == 1:
                     key2type[key] = self.DEFAULT_TYPE_MAP.get(item_types.pop())
@@ -53,17 +63,6 @@ class GeneralCollateFn(object):
             self.update_type_mapping(key2type)
         return key2type
 
-    def _validate_data(self, data: List[dict]):
-        """Validate if all data have the same keys"""
-        all_keys = set()
-        for d in data:
-            if len(all_keys) == 0:
-                all_keys.update(d.keys())
-            else:
-                assert (
-                    set(d.keys()) == all_keys
-                ), "Data instances does not have the same keys!"
-
     def update_data(self, data: List[dict]) -> List[dict]:
         """For those who transform data while collating, override this function"""
         return data
@@ -72,22 +71,31 @@ class GeneralCollateFn(object):
         return data
 
     def __call__(self, data: List[Dict[str, Any]]) -> Dict[str, Any]:
-        self._validate_data(data)
+        validate_instance_has_the_same_keys(data)
         if len(self.key2type) == 0 and self.guessing:
             self.guess_types(data[0], update=True)
 
+        if self.missing_key_as_null:
+            for d in data:
+                missing_keys = set(d.keys()) - set(self.key2type.keys())
+                for mkey in missing_keys:
+                    self.key2type[mkey] = None
+
         data = self.update_data(data)
-
-        final_data = {key: [] for key in self.key2type}
-        for d in data:
-            for key in self.key2type:
-                final_data[key].append(d[key])
-
+        final_data = group_instances_into_batch(data, self.key2type.keys())
         final_data = self.update_before_tensorify(final_data)
 
         for key, val_type in self.key2type.items():
             if val_type is not None and all(val is not None for val in final_data[key]):
-                final_data[key] = torch.tensor(final_data[key], dtype=val_type)
+                if isinstance(final_data[key], torch.Tensor):
+                    if final_data[key].dtype != val_type:
+                        final_data[key] = final_data[key].to(val_type)
+                elif isinstance(final_data[key][0], torch.Tensor):
+                    final_data[key] = torch.stack(final_data[key], dim=0)
+                    if final_data[key].dtype != val_type:
+                        final_data[key] = final_data[key].to(val_type)
+                else:
+                    final_data[key] = torch.tensor(final_data[key], dtype=val_type)
 
         return final_data
 
