@@ -1,6 +1,6 @@
-from typing import Iterable, Optional
+from typing import Iterable, Iterator, Optional
 
-from torch.utils.data import Dataset
+from torch.utils.data import Dataset, IterableDataset
 
 
 class CachedDataset(Dataset):
@@ -47,3 +47,22 @@ class StreamTransformDataset(Dataset):
 
     def __len__(self) -> int:
         return len(self.data)
+
+
+class StreamReadDataset(IterableDataset):
+    def __init__(
+        self, data_iterator: Iterator, transform, debug: Optional[bool] = False
+    ) -> None:
+        super().__init__()
+
+        self.transform = transform
+        self.data_iterator = data_iterator
+        self.debug = debug
+        self.cnt = 0
+
+    def __iter__(self):
+        for item in self.data_iterator:
+            if self.debug and self.cnt >= 500:
+                raise StopIteration
+            yield self.transform(item)
+            self.cnt += 1
