@@ -1,31 +1,31 @@
 from datetime import datetime
 from pathlib import Path
-from typing import List, Optional, Tuple
+from typing import List, Optional, Tuple, Union
 
 import torch
-from omegaconf import OmegaConf
+import torch.nn as nn
+from omegaconf import DictConfig
 
 from rex import accelerator
 from rex.data.data_manager import DataManager
 from rex.data.transforms.base import TransformBase
 from rex.metrics import safe_division
 from rex.tasks.base_task import TaskBase
+from rex.utils.config import DefaultBaseConfig
 from rex.utils.dict import get_dict_content
 from rex.utils.io import dump_json
 from rex.utils.logging import logger
-from rex.utils.progress_bar import pbar, rbar
-from rex.utils.registry import register
-from rex.utils.tensor_move import move_to_device
+from rex.utils.param import calc_module_params
+from rex.utils.progress_bar import pbar
 from rex.utils.wrapper import safe_try
 
 
-@register("task")
 class SimpleTask(TaskBase):
     """Simple task with one model, one optimizer and (maybe) one lr scheduler"""
 
     def __init__(
         self,
-        config: OmegaConf,
+        config: Union[DefaultBaseConfig, DictConfig],
         initialize: Optional[bool] = True,
         makedirs: Optional[bool] = True,
         dump_configfile: Optional[bool] = True,
@@ -37,50 +37,53 @@ class SimpleTask(TaskBase):
         self.measures_path = Path(config.task_dir).joinpath("measures")
         self.measures_path.mkdir(parents=True, exist_ok=True)
 
+        self.optimizer = None
+        self.lr_scheduler = None
+        self.transform: TransformBase
+        self.data_manager: DataManager
+        self.model: nn.Module
+
+        self.initialize()
+        self.after_initialization()
+
+    def initialize(self):
         logger.debug("Init transform")
         self.transform: TransformBase = self.init_transform()
+        logger.debug(f"transform: {type(self.transform)}")
         logger.debug("Init data_manager")
         self.data_manager: DataManager = self.init_data_manager()
+        logger.debug(f"data manager: {type(self.data_manager)}")
 
         logger.debug("Init model")
         self.model = self.init_model()
-
-        if self.config.skip_train:
-            self.optimizer = None
-            self.lr_scheduler = None
-        else:
-            logger.debug("Init optimizer")
-            self.optimizer = self.init_optimizer()
-            logger.debug("Init lr_scheduler")
-            self.lr_scheduler = self.init_lr_scheduler()
-
-        self.after_initialized()
-        logger.debug("SimpleTask initialised")
-
-    def after_initialized(self):
+        logger.debug(f"model: {type(self.model)}")
+        logger.debug(f"model: {self.model}")
+        num_model_params = calc_module_params(self.model)
+        logger.debug(f"#ModelParams: {num_model_params}")
         logger.debug("Prepare model")
-        self.model = accelerator.prepare(self.model)
-        logger.debug("Prepare optimizer")
-        self.optimizer = accelerator.prepare(self.optimizer)
-        if self.lr_scheduler is not None:
-            logger.debug("Prepare lr_scheduler")
-            self.lr_scheduler = accelerator.prepare(self.lr_scheduler)
+        self.model = accelerator.prepare_model(self.model)
 
-    def init_transform(self):
+    def after_initialization(self):
+        pass
+
+    def init_transform(self) -> TransformBase:
         raise NotImplementedError
 
-    def init_data_manager(self):
+    def init_data_manager(self) -> DataManager:
         raise NotImplementedError
 
-    def init_model(self):
+    def init_model(self) -> torch.nn.Module:
         raise NotImplementedError
 
-    def init_optimizer(self):
+    def init_optimizer(self) -> torch.optim.Optimizer:
         raise NotImplementedError
 
-    def init_lr_scheduler(self):
+    def init_lr_scheduler(self) -> Union[None, torch.optim.lr_scheduler._LRScheduler]:
         return None
 
+    def after_whole_train(self):
+        pass
+
     @torch.no_grad()
     def _get_eval_results_impl(
         self, input_batches: List, output_batches: List, *args, **kwargs
@@ -115,15 +118,37 @@ class SimpleTask(TaskBase):
         loader = self.data_manager.load_loader(
             dataset_name, is_eval=is_eval, epoch=epoch
         )
-        loader = accelerator.prepare(loader)
         return loader
 
-    @safe_try
+    def log_loss(
+        self, idx: int, loss_item: float, step_or_epoch: str, dataset_name: str
+    ):
+        pass
+
+    def log_metrics(
+        self, idx: int, metrics: dict, step_or_epoch: str, dataset_name: str
+    ):
+        pass
+
+    # @safe_try
     def train(self):
         if self.config.skip_train:
             raise RuntimeError(
                 "Training procedure started while config.skip_train is True!"
             )
+        else:
+            logger.debug("Init optimizer")
+            self.optimizer = self.init_optimizer()
+            logger.debug(f"optimizer: {self.optimizer}")
+            logger.debug("Prepare optimizer")
+            self.optimizer = accelerator.prepare_optimizer(self.optimizer)
+            logger.debug("Init lr_scheduler")
+            self.lr_scheduler = self.init_lr_scheduler()
+            if self.lr_scheduler is not None:
+                logger.debug(f"lr_scheduler: {type(self.lr_scheduler)}")
+                logger.debug("Prepare lr_scheduler")
+                self.lr_scheduler = accelerator.prepare_scheduler(self.lr_scheduler)
+
         if self.config.resumed_training_path is not None:
             self.load(
                 self.config.resumed_training_path,
@@ -139,6 +164,7 @@ class SimpleTask(TaskBase):
         total_steps = self.history["curr_epoch"] * len(train_loader)
         start_time = datetime.now()
         for epoch_idx in range(self.history["curr_epoch"], self.config.num_epochs):
+            logger.info(f"Start training {epoch_idx}/{self.config.num_epochs}")
             if not resumed_training:
                 self.history["curr_epoch"] = epoch_idx
 
@@ -169,10 +195,13 @@ class SimpleTask(TaskBase):
 
                 result = self.model(**batch)
 
-                loss = result["loss"] / self.config.grad_accum_steps
-                loader.set_postfix({"loss": loss.item()})
-                self.history["current_train_loss"] += loss.item()
-                accelerator.backward(loss)
+                result["loss"] /= self.config.grad_accum_steps
+                accelerator.backward(result["loss"])
+                loss_item = result["loss"].item()
+                self.history["current_train_loss"]["epoch"] += loss_item
+                self.history["current_train_loss"]["step"] += loss_item
+                loader.set_postfix({"loss": loss_item})
+                self.log_loss(self.history["total_steps"], loss_item, "step", "train")
 
                 if self.config.max_grad_norm > 0:
                     accelerator.clip_grad_norm_(
@@ -188,12 +217,16 @@ class SimpleTask(TaskBase):
 
                 if (
                     self.config.step_eval_interval > 0
-                    and (total_steps + 1) % self.config.step_eval_interval
+                    and (self.history["total_steps"] + 1)
+                    % self.config.step_eval_interval
+                    == 0
                 ):
                     self._eval_during_train("step")
                     if not self._check_patience():
                         break
                 total_steps += 1
+                if not resumed_training:
+                    self.history["total_steps"] += 1
 
             logger.info(loader)
             if (self.config.epoch_eval_interval > 0) and (
@@ -221,6 +254,13 @@ class SimpleTask(TaskBase):
             test_loss, test_measures = self.eval(
                 "test", verbose=True, dump=True, postfix="final"
             )
+            self.log_loss(0, test_loss, "final", "test")
+            self.log_metrics(0, test_measures, "final", "test")
+            return test_loss, test_measures
+
+        self.after_whole_train()
+
+        return self.history["best_loss"], self.history["best_metric"]
 
     def _eval_during_train(self, eval_on: Optional[str] = "epoch"):
         """Evaluation during training to record eval info, and control training process
@@ -228,68 +268,79 @@ class SimpleTask(TaskBase):
         Args:
             eval_on: epoch or step
         """
-        # validate
-        assert eval_on in [
-            "epoch",
-            "step",
-        ], f"eval_on: {eval_on} must eq `epoch` or `step`"
-        assert self.config.select_best_by_key in [
-            "metric",
-            "loss",
-        ], f"select_best_by_key is {self.config.select_best_by_key}, while candidates are: `metric` and `loss`"
-        assert (self.config.select_best_by_key == "train") or (
-            self.config.select_best_by_key != "train"
-            and self.config.select_best_on_data in self.config.eval_on_data
-        ), f"{self.config.select_best_on_data} is not included in eval_on_data: {self.config.eval_on_data}"
-
-        if len(self.config.eval_on_data) < 1:
-            logger.info(
-                "Does not provide any data to evaluate during training, continue training",
-                level="WARNING",
-            )
-            return True
-
-        # init
         curr_epoch_idx = self.history["curr_epoch"]
         curr_total_steps = self.history["total_steps"]
-        history_index = curr_epoch_idx if eval_on == "epoch" else curr_total_steps
-        history_index_identifier = f"{eval_on}.{history_index}"
+        history_idx = curr_epoch_idx if eval_on == "epoch" else curr_total_steps
+        history_idx_identifier = f"{eval_on}.{history_idx}"
+        logger.info(f"Start evaluating at {history_idx_identifier}")
+
+        # make sure the model save at least one checkpoint no matter
+        #   evaluation or not
+        if self.config.save_every_ckpt:
+            self.save_ckpt(f"{history_idx_identifier}")
+
+        # validate
+        if eval_on not in ["epoch", "step"]:
+            raise ValueError(f"eval_on: {eval_on} must eq `epoch` or `step`")
+        if self.config.select_best_by_key not in ["metric", "loss"]:
+            raise ValueError(
+                f"select_best_by_key is {self.config.select_best_by_key}, while candidates are: `metric` and `loss`"
+            )
+        if self.config.select_best_on_data not in self.config.eval_on_data:
+            raise ValueError(
+                f"{self.config.select_best_on_data} is not included in eval_on_data: {self.config.eval_on_data}"
+            )
+        if len(self.config.eval_on_data) < 1:
+            logger.warning(
+                "Does not provide any data to evaluate during training",
+            )
+
+        # init
         this_eval_result = {}  # to dump results
 
         # eval to get measurements and loss
         eval_on_datasets = set()
         for dataset_name in self.config.eval_on_data:
             dataset_name = DataManager._get_normalized_dataset_name(dataset_name)
-            eval_on_datasets.add(dataset_name)
+            if dataset_name != "train":
+                eval_on_datasets.add(dataset_name)
+            elif self.config.select_best_by_key != "loss":
+                raise ValueError(
+                    "Only `select_best_by_key=loss` is allowed when `eval_on_data=train`"
+                )
         for dataset_name in eval_on_datasets:
             eval_loss, eval_measures = self.eval(
-                dataset_name, verbose=False, postfix=history_index_identifier
+                dataset_name, verbose=False, postfix=history_idx_identifier
             )
-            self.history[eval_on][dataset_name]["metrics"][
-                history_index
-            ] = eval_measures
-            self.history[eval_on][dataset_name]["loss"][history_index] = eval_loss
+            self.log_loss(history_idx, eval_loss, eval_on, dataset_name)
+            self.log_metrics(history_idx, eval_measures, eval_on, dataset_name)
+            self.history[eval_on][dataset_name]["metrics"][history_idx] = eval_measures
+            self.history[eval_on][dataset_name]["loss"][history_idx] = eval_loss
             this_eval_result[f"{eval_on}.{dataset_name}.metrics"] = eval_measures
             this_eval_result[f"{eval_on}.{dataset_name}.loss"] = eval_loss
 
-        self.history[eval_on][dataset_name]["loss"][history_index] = self.history[
-            "current_train_loss"
-        ]
-        this_eval_result["train_loss"] = self.history["current_train_loss"]
-
         # update the best
         select_best_on_data = DataManager._get_normalized_dataset_name(
             self.config.select_best_on_data
         )
-        metric = self.history[eval_on][select_best_on_data]["metrics"][history_index]
-        metric = get_dict_content(metric, self.config.best_metric_field)
+        metric = -1.0
         is_best_metric = False
-        if metric > self.history["best_metric"]:
-            is_best_metric = True
-            self.history["best_metric"] = metric
-        this_eval_result["is_best_metric"] = is_best_metric
+        if not (
+            select_best_on_data == "train" and self.config.select_best_by_key == "loss"
+        ):
+            metric = self.history[eval_on][select_best_on_data]["metrics"][history_idx]
+            metric = get_dict_content(metric, self.config.best_metric_field)
+            if metric > self.history["best_metric"]:
+                is_best_metric = True
+                self.history["best_metric"] = metric
+            this_eval_result["is_best_metric"] = is_best_metric
 
-        loss = self.history[eval_on][select_best_on_data]["loss"][history_index]
+        this_eval_result["train_loss"] = self.history["current_train_loss"][eval_on]
+        if select_best_on_data == "train" and self.config.select_best_by_key == "loss":
+            loss = this_eval_result["train_loss"]
+            self.log_loss(history_idx, loss, eval_on, "sum_train_loss")
+        else:
+            loss = self.history[eval_on][select_best_on_data]["loss"][history_idx]
         is_best_loss = False
         if loss < self.history["best_loss"]:
             is_best_loss = True
@@ -321,14 +372,12 @@ class SimpleTask(TaskBase):
 
         # print results
         logger.info(
-            f"Eval on {eval_on}, Idx: {history_index_identifier}, is_best: {is_best}"
+            f"Eval on {eval_on}, Idx: {history_idx_identifier}, is_best: {is_best}"
         )
-        logger.info(f"Train loss: {self.history['current_train_loss']:.5f}")
+        logger.info(f"Train loss: {this_eval_result['train_loss']:.5f}")
         for dataset_name in eval_on_datasets:
-            eval_measures = self.history[eval_on][dataset_name]["metrics"][
-                history_index
-            ]
-            eval_loss = self.history[eval_on][dataset_name]["loss"][history_index]
+            eval_measures = self.history[eval_on][dataset_name]["metrics"][history_idx]
+            eval_loss = self.history[eval_on][dataset_name]["loss"][history_idx]
             logger.info(
                 (
                     f"{dataset_name} - "
@@ -348,18 +397,19 @@ class SimpleTask(TaskBase):
         )
         dump_json(
             this_eval_result,
-            self.measures_path.joinpath(f"{history_index_identifier}.json"),
+            self.measures_path.joinpath(f"{history_idx_identifier}.json"),
             indent=2,
         )
 
         # reset current training loss
-        self.history["current_train_loss"] = 0.0
+        self.history["current_train_loss"][eval_on] = 0.0
 
         # save checkpoints
-        if self.config.save_every_ckpt:
-            self.save_ckpt(f"{history_index_identifier}")
-        if is_best and self.config.save_best_ckpt:
-            self.save_ckpt("best")
+        if is_best:
+            if self.config.save_best_ckpt:
+                self.save_ckpt("best")
+            if self.config.save_best_ckpt == "all":
+                self.save_ckpt(f"best.{history_idx_identifier}")
 
     def _check_patience(self):
         """Check patience, returns False if training process should be stopped, else returns True"""
@@ -408,9 +458,7 @@ class SimpleTask(TaskBase):
         eval_loader = self.get_data_loader(
             dataset_name, is_eval=True, epoch=self.history["curr_epoch"]
         )
-        loader = pbar(
-            eval_loader, desc=f"{dataset_name} - {postfix} Eval", ncols=80, ascii=True
-        )
+        loader = pbar(eval_loader, desc=f"{dataset_name} - {postfix} Eval", ascii=True)
 
         eval_loss = 0.0
         metrics = {}
@@ -418,14 +466,10 @@ class SimpleTask(TaskBase):
         output = []
         # raw_batch: dict
         for batch in loader:
-            out = self.model(**batch)
+            out = self.model(**batch, is_eval=True)
             eval_loss += out["loss"].item()
-            batch = move_to_device(batch, "cpu")
-            out = move_to_device(out, "cpu")
-            all_batch = accelerator.gather(batch)
-            all_out = accelerator.gather(out)
-            origin.append(all_batch)
-            output.append(all_out)
+            origin.append(batch)
+            output.append(out)
 
         logger.info(loader)
         metrics = self._get_eval_results_impl(origin, output, postfix)
