@@ -1,18 +1,13 @@
 import os
 
+# pip install datasets
 from datasets import load_dataset
 
 from rex.data.label_encoder import LabelEncoder
-from rex.utils.build_emb import build_emb
-from rex.utils.io import dump_iterable, dump_jsonlines
+from rex.utils.io import dump_iterable, dump_json, dump_jsonlines
 
 cache_dir = "./data/cache"
 dump_dir = "./data/formatted"
-# downloaded emb file from https://github.com/Embedding/Chinese-Word-Vectors
-input_embedding_filepath = (
-    "/data4/tzhu/CCKS2020-FinEBody-nonNaN/data/sgns.sogounews.bigram-char"
-)
-output_embedding_filepath = "./data/formatted/vocab.emb"
 
 datasets = load_dataset("peoples_daily_ner", cache_dir=cache_dir)
 lbe = LabelEncoder()
@@ -44,4 +39,12 @@ for key in datasets:
     dump_iterable(tokens, vocab_filepath)
 
 lbe.save_pretrained(os.path.join(dump_dir, "label2id.json"))
-build_emb(input_embedding_filepath, output_embedding_filepath, *vocab_files)
+
+dump_json(
+    {
+        "LOC": "地名地理位置",
+        "PER": "人名公民居民老百姓名人明星",
+        "ORG": "公司法院企业集团学校医院单位",
+    },
+    os.path.join(dump_dir, "role2query.json"),
+)
